import os
import yaml
import functools
from groAggregator.connector import Connector

class Executor(Connector):
    def __init__(self, dbConfig, instance, db=None):
        super().__init__(dbConfig)
        self.instance = instance
        self.db = db

        self.conn = None
        self.cursor = None

        self.__LOCATION__ = os.path.dirname(__file__)
        #self.CONFIG_commands = self.CONFIG['commands']
        #self.CONFIG_eventTableMapping = self.CONFIG['events']

    def __enter__(self):
        # This ensure, whenever an object is created using "with"
        # this magic method is called, where you can create the connection.
        self.conn = self.get_connections(self.instance, self.db)
        self.cursor = self.conn.cursor()
        return self
    
    def __exit__(self, exception_type, exception_val, trace):
        # once the with block is over, the __exit__ method would be called
        # with that, you close the connnection
        try:
           self.cursor.close()
           self.conn.close()
        except AttributeError: # isn't closable
           print('DB connector not closable.')
           return True # exception handled successfully

    # func argument is used as a key to map to return 
    @functools.lru_cache(maxsize=256)
    def cacheWrapper(self, command) -> list:
        self.cursor.execute(command)
        colnames = [desc[0] for desc in self.cursor.description]
        data = self.cursor.fetchall()
        
        namedDictData = [dict(zip(colnames, d)) for d in data]
        return namedDictData

    def createCommand(self, commandTemplate, events, **kwargs):
        command = self.CONFIG_commands[commandTemplate]
        result = []
        for event in events:
            args = {**self.CONFIG_eventTableMapping[event], **kwargs}
            result.append({event: command.format(**args)})
        return result
    
    def retreive(self, command) -> dict:
        event = list(command.keys())[0]
        command = list(command.values())[0]
        result = {event: self.cacheWrapper(command)}
        print(result)
        return {event: self.cacheWrapper(command)}

        
        

        
        

