import os
import yaml
import functools
import operator
import itertools
from gro_aggregation.executor import Executor

'''
    - right now Aggregator would execute by : 
        1. taking a definition file
        2. apply arguments from frontend if any
        3. execute sequentially
    - chaining previous deinitions are available, eg: getNetDepositsTokenAddress
'''
class Aggregator(Executor):
    def __init__(self, instance=None, db=None):
        super().__init__(instance, db)

        self.__LOCATION__ = os.path.dirname(__file__)

    # use a simple map to save exact same action with exact same kwargs
    @functools.lru_cache(maxsize=256)
    def aggregate(self, action, **kwargs) -> list:
        with open(os.path.join(self.__LOCATION__, 'definition/{}.yaml'.format(action)), 'r') as f:
            actionDict = yaml.safe_load(f)['definition']

        c = actionDict['consolidate']
        # chain is using other aggregation rountines 
        if 'chain' in actionDict:
            print("chaining : ",action, " ", actionDict)
            aggregatedResult = [self.aggregate(i, **kwargs) for i in actionDict['chain']]
            return getattr(self, c)(aggregatedResult)
            
        command = actionDict['command']
        fMap = actionDict['aggregate']
        # create SQL commands from commands and argument passed from frontend
        args = {**actionDict, **kwargs}
        # take action to the executor context
        command = {action: command.format(**args)}
        # execute SQL or extract from cache
        data = self.retreive(command)
        # apply aggregate functions within each table
        # aggregate set to None if no aggregation is needed, eg getTransactions
        if fMap != 'None':
            f = list(fMap.keys())[0]
            v = list(fMap.values())[0]
            aggregatedData = getattr(self, f)([data], **v)
        else:
            aggregatedData = data
        
        # apply consolidate function on aggregated data from tables
        return getattr(self, c)(aggregatedData)

    # aggregate operations, expect to take {"fields" : []} in kwargs
    def sum(self, resultList, **kwargs) -> list:
        # TODO maybe make a list comprehension 
        result = []
        for event in resultList:
            eventData = list(event.values())[0]
            field = kwargs["fields"]
            result.append(float(sum([i[field] for i in eventData])))
        return result
            
    def average(self, resultList, **kwargs) -> list:
        # TODO maybe make a list comprehension 
        result = []
        for event in resultList:
            eventData = list(event.values())[0]
            field = kwargs["fields"]
            result.append(float(sum([i[field] for i in eventData]))/ len(eventData))
        return result


    # reduce operation
    def returns(self, aggregatedResult) -> list:
        return aggregatedResult

    def offset(self, aggregatedResult) -> float:
        return functools.reduce(lambda x, y: x[0]-y[0], aggregatedResult)

    def accumulate(self, aggregatedResult) -> float:
        # [1,2,3] -> [1.3,6]
        return list(itertools.accumulate(aggregatedResult,func=operator.add))[-1]